# OVS Service Function chain for HBN


# Change Log

## Enhancements added for Jan 2023 internal Release

* Support for own namespace for HBN
* CNI configuration for container primary interface
* br-mgmt bridge for connecting HBN container to the outside via mgmt vrf
* Source NAT for HBN to communicate outside be oob_net0 address
* Port forwarding for HBN NVUE Rest API port.


## Design/Functional Spec

Please refer [Document - HBN Own namespace design](https://docs.google.com/document/d/1GJtcjvZTsrObUHacG9zk5qDGFs_oxHrxnNtOSaE6ATw/edit#)
for more information.

# Installation Instructions

## Automated Installation

Run the included script `install.sh` with the number of SFs needed on each ECPF. It will do the following:
* Turn off automatic bridge creation in `/etc/mellanox/mlnx-ovs.conf`
* Generate required SF configs with mac address (in `/etc/mellanox/mlnx-sf.conf`)
* Generate `sfc.conf` with SFs requested and place it in `/etc/mellanox/sfc.conf`
* Copy udev SF renaming scripts to udev scripts location `/usr/lib/udev/`
* Create systemd service file `sfc.service` and place it in `/etc/systemd/system/sfc.service`
* Place the OVS SFC logic in `/opt/mellanox/sfc/sfc.sh` and make it executable
* Enable systemd service
* Copy 50-ifup-hooks into /etc/networkd-dispatcher/routable.d to configure management VRF interface
* Run nvconfig commands
* [Optional] Reboot DPU

Steps added for Jan 2023 release
* Additional SFs for DPU services to HBN communication
* Configure default CNI
* Additional IPtables rules to enable NAT/Port forward via mgmt vrf.


Use `./install.sh -h | --help` for usage details.

### Notes on the install script
  * Script has only been tested with Ubuntu BFB image.
  * SFs are now configured with -t option. Use Jan 2023 GA BFB image or newer.
  * Atleast 6 SFs are created if you run `install.sh` without any options. These are mapped to the uplink ports (`p0` and `p1`)
    and PF representors (`pf0hpf` and `pf1hpf`) as well as DPU specific SFs (`dpu0_sf` and `dpu1_sf`). The SFs mapped to
    the uplink ports are configured to use the  same mac address as `p0` and `p1`.
  * The number of sfs on each ECPF can be controlled by the optional arguments `-p0 <num>` and `-p1 <num>`.
    If one of them is skipped, then the default value of 3 is used. (used for mapping uplink and PF representor).
    These can be controlled via the bfb.cfg configuration file.  In the bfb.cfg file set the following.

    ```
    ENABLE_SFC_HBN=yes
    NUM_SFs_ECPF0=19 # (the default value)
    NUM_SFs_ECPF1=3 # (the default value)
    # bfb-install  -c bf.cfg -r rshim0 -b <BFB image path>
    ```
  * Auto generated port mappings can be seen at `/etc/mellanox/sfc.conf`.
  * The script assumes that the HBN container would be brought up kubelet.

## Manual Installation   ( No longer recommended)

### Turn off Default OVS bridge creation

- Open `/etc/mellanox/mlnx-ovs.conf` and change CREATE_OVS_BRIDGES="no"

### Scalable Functions

- Create SFs using  `/etc/mellanox/mlnx-sf.conf`. Set the device pci address and sfnum index for each device.

```bash
cat <<EOF | tee /etc/mellanox/mlnx-sf.conf
/sbin/mlnx-sf --action create --device 0000:03:00.0 --sfnum 0 --hwaddr 02:5a:49:f1:a5:2a
/sbin/mlnx-sf --action create --device 0000:03:00.0 --sfnum 1 --hwaddr 02:5a:c9:e3:1c:d3
/sbin/mlnx-sf --action create --device 0000:03:00.0 --sfnum 2 --hwaddr 02:5a:c9:d2:cc:b6
/sbin/mlnx-sf --action create --device 0000:03:00.0 --sfnum 3 --hwaddr 02:5a:c9:4d:9c:7f
/sbin/mlnx-sf --action create --device 0000:03:00.1 --sfnum 0 --hwaddr 02:9c:14:12:d4:ce
/sbin/mlnx-sf --action create --device 0000:03:00.1 --sfnum 1 --hwaddr 02:9c:8d:0b:b9:c9
/sbin/mlnx-sf --action create --device 0000:03:00.1 --sfnum 2 --hwaddr 02:9c:25:06:f7:38
/sbin/mlnx-sf --action create --device 0000:03:00.1 --sfnum 3 --hwaddr 02:9c:3b:55:91:05
EOF
```

### Systemd Service files

```bash
cat <<EOF | tee /etc/systemd/system/sfc.service
[Unit]
Description=Setup SFC
After=openvswitch-switch.service
Requires=openvswitch-switch.service
PartOf=openvswitch-switch.service

[Service]
Type=oneshot
ExecStart=/opt/mellanox/sfc/sfc.sh
RemainAfterExit=true
StandardOutput=journal

[Install]
WantedBy=multi-user.target
EOF
```

### OVS SFC config file

- See the provided sample config file `sfc.conf.example`. Adjust the contents of `sfc.conf.example` as neccessary.
- Copy the contents of `sfc.conf.example` file to  `/etc/mellanox/sfc.conf`

```bash
cp /path/to/this/repo/sfc.conf.example /etc/mellanox/sfc.conf
```

### Place OVS SFC Logic

```bash
mkdir -p /opt/mellanox/sfc
cp /path/to/this/repo/sfc.sh  /opt/mellanox/sfc/
chmod +x /opt/mellanox/sfc/
```
### Enable service

```bash
systemctl daemon-reload
systemctl enable sfc
```

### Review static pod example

The repo contains an sample yaml for a static pod in the host namespace in the file `netshoot-static.yaml`. Use this as a template and adjust the contents as per the HBN container requirements.

When ready copy it to `/etc/kubelet.d/`

```bash
cp <path/to/hbn_container_pod.yaml> /etc/kubelet.d/
```

### Reboot the DPU

```bash
reboot
```

# Usage

You can see the status of the sfc service from systemd

```bash
systemctl status sfc
journalctl -u sfc   # go to end of the logs with key `G`
```

## Interacting with SFs

Use the tool `mlnx-sf` to get details and interact with SF devices.
- To list SFs
```bash
mlnx-sf -a show
```

- To create
```bash
# Format: /sbin/mlnx-sf --action create --device <pci address> --sfnum <index> --hwaddr <mac address>
/sbin/mlnx-sf --action create --device 0000:03:00.0 --sfnum 0 --hwaddr 02:5a:49:f1:a5:2a
```

- To delete
```bash
# Format: /sbin/mlnx-sf -d <pci address> -a delete -n <sf index num> --sfindex <rep netdev>  # Use `mlnx-sf -a show` and use 'Representor netdev'
/sbin/mlnx-sf -d 0000:03:00.1 -a delete -n 3 --sfindex en3f1pf1sf3
```

